// fanta_presets: gFantasticLib.js must be loaded before this file
var fanta_presets = new function() {
	this.moduleId="fap";	// module id
	this.moduleIcon=baseImgUrl+"fanta_presets_icon.png";
	
	this.thumbs=new Array();
	this.cats = new Array();
	
	var camera = new FSCamera();
	var offset = new FSOffset();
	var orientation = new FSOrientation(0);
	
	var tCount = 0;
	var tWidth = 128;
	var tHeight = 128;
	var tScaleX =1.2;
	var tScaleY = 0.6;
	var screenWidth = 320;
	var screenHeight = 480;
	var rowHeight = tHeight*tScaleY+5;
	var colWidth = tWidth*tScaleX+5;
	var xmax=0;
	var xoff = 0;
	var xo = 0;
	var timerOn = 0;
	var inited=false;
	var timer = false;
	
	var currPresetCatIndex = 0;
	var catindex = 0;
	var miny = -30;
	var maxy = 1000;
	var maxx = 1000;
	var maxys = new Array();
	var sx,sy,lx,ly,cx,cy;
	var t1 = undefined;
	var t2 = undefined;
	var began = false;
	
	var modHead = new FSModel("fpModHead");
	var modHeadText = new FSModel("fpModHeadText",0,modHead);
	var modHeadX = 160;
	var modHeadY = -10;
	
		
	this.init = function() {
		fs.send('fanta_presets_init 1');
		inited = true;
	};
	
	
	this.activate = function() {
		camera.updatePosition();
		offset.updatePosition();
		orientation.updateOrientation();
		
		if (!inited) {
			this.init();
		} else {
			for (var t in this.thumbs) {
				this.thumbs[t].visible = true;
			}
			modHead.visible = true;
		}
	};
	
	this.deactivate = function() {
		if (inited) {
			for (var t in this.thumbs) {
				this.thumbs[t].visible = false;
			}
			modHead.visible = false;
		}
		if (t1 != undefined) t1.stop();
		if (t2 != undefined) t2.stop();
	};
	
	this.setThumbCount = function(c) {
		tCount = c;
		xmax = ( ( (Math.floor(tCount/3) + 1) * colWidth) - screenWidth);
		
		for (var i=0; i<tCount; i++) {
			var m = new FSModel("fapthumb"+i,128);
			this.thumbs[m.name] = m;
			m._scaleX = tScaleX;
			m.scaleY = tScaleY;
		}
		
		modHead.boundsWidth = 320;
		modHead.boundsHeight = 80;
		modHead._x = modHeadX;
		modHead._y = modHeadY;
		modHead.z = 1;
		modHead.xyz = new Array(-160,-15,0,  160,-15,0,  160,15,0,  -160,15,0);
		modHead.renderMode = "triangle_fan";
		modHead._r = 1; m._g = 0; m._b = 0;
		modHead.a = 0.5;
		
		modHeadText._y = 10;
		modHeadText._x = -150;
		modHeadText.z = 2; // ! important
		modHeadText.font = "American Typewriter";
		modHeadText.fontSize = 18;
		modHeadText._r = 1;
		modHeadText._g = 1;
		modHeadText.b = 1;
		modHeadText.text =" ";
		
	}
	
	this.updateThumbs = function() {
		var yyy = 0;
		var yyyy = 0;
		maxx = screenWidth*(this.cats.length-1);
		
		maxys = new Array();
		
		modHeadText.text = this.cats[0];
		
		for (var c=0; c<this.cats.length; c++) {
			var count = 0;
			for (var t in this.thumbs) {
				thumb = this.thumbs[t];
				if (thumb.vcat == this.cats[c]) {
					thumb.x = colWidth/2 + colWidth*(count%2) + c*screenWidth;
					thumb.y = rowHeight/2 + rowHeight*Math.floor(count/2);
					thumb.visible = true;
					
					if (thumb.y>yyy) yyy=thumb.y;
					count++;
					
					/*var ttext = new FSModel("fappostfx"+t,0,this.thumbs[t]);
					ttext.text = thumb.vpostfx;
					ttext.font = "Arial Rounded MT Bold";
					ttext.y = 39;
					ttext.x = -70;
					ttext.z = 1;*/
				}
			}
			if (yyy>yyyy) yyyy = yyy;
			maxys[c] = Math.max(0,yyy-screenHeight+rowHeight/2);
			yyy = 0;
		}
		
		maxy = Math.max(miny,yyyy-screenHeight+rowHeight/2);
		
		
		camera.y = miny;
	};
	
	this.touch = function(type, x, y, id, area) {
		if (id == "1") {
			if (type == "B") {
				lx = x; ly = y;
				sx = x; sy = y;
				cx = camera.x;
				cy = camera.y;
				began = true;
			}
			else if (type == 'M') {
				lx = x; ly = y;
				
				var newx = cx + (sx-x);	// X
				if (newx<miny) newx *= 0.5;
				if (newx>maxx) newx = newx + (maxx-newx)/2;
				
				if (t2 != undefined) t2.stop();	// X
				t2 = new Tween(camera,'_x',Tween.strongEaseOut,camera.x,newx,0.1);
				t2.start();
				
				var newy = cy + (sy-y); 	// Y
				if (newy<miny) newy *= 0.5;
				if (newy>maxy) newy = newy + (maxy-newy)/2;
				
				if (t1 != undefined) t1.stop();	// Y
				t1 = new Tween(camera,'y',Tween.strongEaseOut,camera.y,newy,0.1);
				t1.start();
			} 
			else if (type == 'E') {
				if (!began) return;
				began = false;
				if (Math.abs(sy-y) < 7 && Math.abs(sx-x) < 7) {
					this.processClick(type,x,y,id,area);
				} else {
					var camy = camera.y;	//Y
					var newy = camy;
					if (camy < miny ) newy = miny;
					if (camy > maxy) newy = maxy;
					
					
					if (newy != camy) { // Y
						if (t1 != undefined) t1.stop();
						t1 = new Tween(camera,'y',Tween.strongEaseOut,camy,newy,0.4);
						t1.start();
					} else {
						if (Math.abs(ly-y) > 7) {
							newy = camy+(ly-y)*20;
							if (newy<miny) newy = miny;
							if (newy>maxy) newy = maxy;
							if (t1 != undefined) t1.stop();
							t1 = new Tween(camera,'y',Tween.strongEaseOut,camy,newy,2);
							t1.start();
						}
					}
					
					var camx = camera.x;
					var idx = Math.floor(camx/screenWidth + 0.5);
					idx = Math.min(idx,this.cats.length-1);
					catindex = idx;
					var newx = idx*screenWidth;
					if (t2 != undefined) t2.stop();
					t2 = new Tween(camera,'x',Tween.strongEaseOut,camx,newx,0.4);
					t2.start();
					maxy = maxys[catindex];
					modHeadText.text = this.cats[catindex];
					if (catindex==currPresetCatIndex) modHeadText.g = modHeadText._b = 0;
					else  modHeadText.g = modHeadText._b = 1;
					
					if (camera.y > maxy) {
						if (t1 != undefined) t1.stop();
						t1 = new Tween(camera,'y',Tween.strongEaseOut,camera.y,maxy,0.1);
						t1.start();
					}
					
				}
			}
		}
	};
	
	var tTween = new Tween();
	this.processClick = function(type,x,y,id,area) {
		var xx = camera.x+x;
		var yy = camera.y+y;
		for (var t in this.thumbs) {
			if (this.thumbs[t].hitTest(xx,yy)) {
				fs.send('CHOOSE '+this.thumbs[t].vindex+' '+this.thumbs[t].vbank);
				currPresetCatIndex = catindex;
				
				new Tween(this.thumbs[t],'a',Tween.regularEaseIn,0.2,0.99999,0.2).start();
				return;
			}
		}
	};
	
	this.camera = function(x,y) {
		modHead.x = modHeadX+x;
	};
};

fanLib.addModule(fanta_presets);



