var baseImgUrl = "$1";

Array.prototype.has = function(value) {
	for (var i = 0; i < this.length; i++) {
		if (this[i] === value) {
			return true;
		}
	}
	return false;
};

Array.prototype.removeValue = function(value) {
	for (var i = 0; i < this.length; i++) {
		if (this[i] === value) {
			this.splice(i,1);
			return i;
		}
	}
	return -1;
};

// FSModel: Encapsulates fs.cmd('model....')
var fsRoot = new function() {
	this.absX = 0;
	this.absY = 0;
	this.absZ = 0;
	this.children = new Array();
	
	this.addChild = function(c) {
		if (! this.children.has(c)) this.children.push(c);
	};
};

/**
 * @class FSModel
 * @param {String} n Name (required)
 * @param {int} size If specified, the bounds for hitTest purposes will be determined in conjunction
 * 					with scaleX,Y,Z. Alternatively, make 0 and set boundsX and boundsY.
 * @param {object} parent parent object, fsRoot if not specified
 */
function FSModel(name,size,parent) {
	this._parent = undefined;
	this.children = new Array();
	if (size == undefined) size=0;
	if (parent == undefined) this.parent=fsRoot;
	else this.parent = parent;
	
	this.name=name;
	this._x=0;
	this._y=0;
	this._z=0;
	this._r=1;
	this._g=1;
	this._b=1;
	this._a=0.99999;
	this._scaleX=1;
	this._scaleY=1;
	this._scaleZ=1;
	this._rotateX=0;
	this._rotateY=0;
	this._rotateZ=0;
	this._image='';
	this.size=size;
	this._xyz=new Array();
	this._visible = 1;
	this._text = "";
	this._font = "Courier";
	this._fontSize = 12;
	this._renderMode = "lines";
	this.boundsWidth = 0;
	this.boundsHeight = 0;
}

FSModel.prototype = {
	get x(){
		return this._x;
	},
	set x(v){
		this._x = v;
		this.updatePosition();
	},
	get y(){
		return this._y;
	},
	set y(v){
		this._y = v;
		this.updatePosition();
	},
	get z(){
		return this._z;
	},
	set z(v){
		this._z = v;
		this.updatePosition();
	},
	get scaleX(){
		return this._scaleX;
	},
	set scaleX(v){
		this._scaleX = v;
		this.updateScale();
	},
	get scaleY(){
		return this._scaleY;
	},
	set scaleY(v){
		this._scaleY = v;
		this.updateScale();
	},
	get scaleZ(){
		return this._scaleZ;
	},
	set scaleZ(v){
		this._scaleZ = v;
		this.updateScale();
	},
	get rotateX(){
		return this._rotateX;
	},
	set rotateX(v){
		this._rotateX = v;
		this.updateRotation();
	},
	get rotateY(){
		return this._rotateY;
	},
	set rotateY(v){
		this._rotateY = v;
		this.updateRotation();
	},
	get rotateZ(){
		return this._rotateZ;
	},
	set rotateZ(v){
		this._rotateZ = v;
		this.updateRotation();
	},
	get image(){
		return this._image;
	},
	set image(s){
		this._image = s;
		this.updateImage();
	},
	get r(){
		return this._r;
	},
	set r(v){
		this._r = v;
		this.updateColor();
	},
	get g(){
		return this._g;
	},
	set g(v){
		this._g = v;
		this.updateColor();
	},
	get b(){
		return this._b;
	},
	set b(v){
		this._b = v;
		this.updateColor();
	},
	get a(){
		return this._a;
	},
	set a(v){
		this._a = v;
		this.updateColor();
	},
	get xyz(){
		return this._xyz;
	},
	set xyz(a){
		this._xyz = a;
		this.updateXYZ();
	},
	get absX(){
		return (this._x + this._parent.absX);
	},
	set absX(v){
		this._x = v - this._parent.absX;
		this.updatePosition();
	},
	get absY(){
		return (this._y + this._parent.absY);
	},
	set absY(v){
		this._y = v - this._parent.absY;
		this.updatePosition();
	},
	get absZ(){
		return (this._z + this._parent.absZ);
	},
	set absZ(v){
		this._z = v - this._parent.absZ;
		this.updatePosition();
	},
	get visible() {
		return (this._visible==1);
	},
	set visible(b) {
		this._visible = b ? 1 : 0;
		this.updateColor();
		for (var i=0; i<this.children.length; i++) this.children[i].visible = b;
	},
	get text(){
		return this._text;
	},
	set text(t){
		this._text = t;
		this.updateText();
	},
	get font(){
		return this._font;
	},
	set font(f){
		this._font = f;
		this.updateFont();
	},
	get fontSize(){
		return this._fontSize;
	},
	set fontSize(v){
		this._fontSize = v;
		this.updateFontSize();
	},
	get renderMode() {
		return this._renderMode;
	},
	set renderMode(s){
		this._renderMode = s;
		this.updateRenderMode();
	},
	get parent() {
		return this._parent;
	},
	set parent(p){
		if (this._parent != undefined) this._parent.children.removeValue(this);
		this._parent = p;
		this._parent.addChild(this);
	},
	get rgba() {
		return [this._r,this._g,this._b,this._a];
	},
	set rgba(a) {
		this._r = a[0];
		this._g = a[1];
		this._b = a[2];
		this.a = a[3];
	}
};

FSModel.prototype.addChild = function(c) {
	if (! this.children.has(c)) this.children.push(c);
}

FSModel.prototype.hitTest = function(x,y) {
	if (this.size>0) {
		// for images
		var xo = this.size*Math.abs(this._scaleX)/2;
		var yo = this.size*Math.abs(this._scaleY)/2;
		return ( (x>this.absX-xo) && (x<this.absX+xo) && (y>this.absY-yo) && (y<this.absY+yo) );
	} else if (this.boundsWidth>0) {
		// for everything else
		var xo = this.boundsWidth/2;
		var yo = this.boundsHeight/2;
		return ( (x>this.absX-xo) && (x<this.absX+xo) && (y>this.absY-yo) && (y<this.absY+yo) );
	}
	return false;
}

FSModel.prototype.updatePosition = function() {
	fs.cmd('model '+this.name+' position '+this.absX+' '+this.absY+' '+this.absZ);
	//fs.cmd('model '+this.name+' position '+this._x+  ' '+this._y+  ' '+this._z);
	for (var i=0; i<this.children.length; i++) this.children[i].updatePosition();
}

FSModel.prototype.updateScale = function() {
	fs.cmd('model '+this.name+' scale '+this._scaleX+' '+this._scaleY+' '+this._scaleZ);
}

FSModel.prototype.updateRotation = function() {
	fs.cmd('model '+this.name+' rotate '+this._rotateX+' '+this._rotateY+' '+this._rotateZ);
}

FSModel.prototype.updateImage = function() {
	fs.cmd('model '+this.name+' image '+this._image);
}

FSModel.prototype.updateColor = function() {
	fs.cmd('model '+this.name+' color '+this._r+' '+this._g+' '+this._b+' '+this._a*this._visible);
}

FSModel.prototype.updateXYZ = function() {
	if (this._xyz.length>0) fs.cmd('model '+this.name+' xyz '+this._xyz.join(' '));
}

FSModel.prototype.updateText = function() {
	fs.cmd('model '+this.name+' text '+this._text);
}

FSModel.prototype.updateFont = function() {
	fs.cmd('model '+this.name+' font '+this._font);
}

FSModel.prototype.updateFontSize = function() {
	fs.cmd('model '+this.name+' fontsize '+this._fontSize);
}

FSModel.prototype.updateRenderMode = function() {
	fs.cmd('model '+this.name+' render '+this._renderMode);
}

FSModel.prototype.clear = function() {
	fs.cmd('clear '+this.name);
}

FSModel.prototype.updateAll = function() {
	//this.updateXYZ();
	this.updateImage();
	this.updateScale();
	this.updatePosition();
	this.updateRotation();
	this.updateColor();
}

// FSCamera: Encapsulates fs.cmd('camera....')
function FSCamera(x,y) {
	if (x==undefined) x=0;
	if (y==undefined) y=0;
	this._x=x;
	this._y=y;
}

FSCamera.prototype = {
	get x(){
		return this._x;
	},
	set x(v){
		this._x = v;
		this.updatePosition();
	},
	get y(){
		return this._y;
	},
	set y(v){
		this._y = v;
		this.updatePosition();
	}
};

FSCamera.prototype.updatePosition = function() {
	fs.cmd('camera '+this._x+' '+this._y);
	fanLib.camera(this._x, this._y);
}

// FSOffset: Encapsulates fs.cmd('offset...')
function FSOffset(x,y) {
	if (x==undefined) x=0;
	if (y==undefined) y=0;
	this._x=x;
	this._y=y;
}

FSOffset.prototype = {
	get x(){
		return this._x;
	},
	set x(v){
		this._x = v;
		this.updatePosition();
	},
	get y(){
		return this._y;
	},
	set y(v){
		this._y = v;
		this.updatePosition();
	}
};

FSOffset.prototype.updatePosition = function() {
	fs.cmd('offset '+this._x+' '+this._y);
	fanLib.offset(this._x, this._y);
}

// FSOrientation: Encapsulates fs.cmd('orientation....')
function FSOrientation(o) {
	if (o==undefined) o=1;
	this._orientation=o;
}

FSOrientation.prototype = {
	get orientation(){
		return this._orientation;
	},
	set orientation(v){
		this._orientation = v;
		this.updateOrientation();
	}
};

FSOrientation.prototype.updateOrientation = function() {
	fs.cmd('orientation '+this._orientation);
	fanLib.orientation(this._orientation);
}

// fanLib: Singleton that manages modules
var fanLib = (function() {
	
	// private
	
	var currModule = undefined;
	
	return { // public interface
		_camera: new FSCamera(),
		_offset: new FSOffset(),
		_orientation: new FSOrientation(),
		modules: new Array(),
		pmodules: new Array(),
		getCurrModule: function() {
			return currModule;
		},
		addModule: function(m, persistent) {
			var reinit = (currModule != undefined && m.moduleId == currModule.moduleId);
			fanLib.removeModule(m, persistent);	// maybe not the best way
			if (!persistent) {
				fanLib.modules.push(m);
			} else {
				fanLib.pmodules.push(m);
			}
			if (reinit) this.activateModule(m);
		},
		removeModule: function(m, persistent) {
			var mods;
			if (persistent) mods = fanLib.pmodules;
			else mods = fanLib.modules;
			for (var i=0;i<mods.length; i++) {
				if (m.moduleId == mods[i].moduleId) mods.splice(i,i);
			}
			if (persistent) fanLib.pmodules = mods;
			else fanLib.modules = mods;
		},
		activateModule: function (m) {
			if (currModule != undefined) currModule.deactivate();
			currModule = m;
			currModule.activate();
		},
		accel: function(x,y,z) {
			for (var i=0;i<fanLib.pmodules.length; i++) {
				if (false == fanLib.pmodules[i].accel(x,y,z)) return;
			}
			if (currModule != undefined) currModule.accel(x,y,z);
		},
		touch: function(type,x,y,id,area) {
			for (var i=0;i<fanLib.pmodules.length; i++) {
				if (false == fanLib.pmodules[i].touch(type,x,y,id,area)) return;
			}
			if (currModule != undefined) currModule.touch(type,x,y,id,area);
		},
		offset: function(x,y) {
			this._offset._x = x;
			this._offset._y = y;
			for (var i=0;i<fanLib.pmodules.length; i++) {
				if (false == fanLib.pmodules[i].offset(x,y)) return;
			}
		},
		camera: function(x,y) {
			this._camera._x = x;
			this._camera._y = y;
			for (var i=0;i<fanLib.pmodules.length; i++) {
				if (false == fanLib.pmodules[i].camera(x,y)) return;
			}
			if (currModule != undefined) currModule.camera(x,y);
		},
		orientation: function(o) {
			this._orientation._orientation = o;
			for (var i=0;i<fanLib.pmodules.length; i++) {
				if (false == fanLib.pmodules[i].orientation(o)) return;
			}
		}
	};
})();

fs.cmd('clearmodels');
fs.cmd('orientation 0');

function accel(x, y, z) {
	fanLib.accel(x,y,z);
}
function touch(type, x, y, id, area) {
	fanLib.touch(type,x,y,id,area);
}