#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "gFantastickReader", Category = "String", Help = "Parse Fantastick Messages", Tags = "Fantastick")]
	#endregion PluginInfo
	public class StringgFantastickReaderNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Input", DefaultString = "")]
		ISpread<string> fInput;
		
		[Input("Filters", DefaultString = "")]
		ISpread<string> fFilters;
		
		[Output("Output")]
		ISpread<string> FOutput;
		
		[Output("Filtered Input")]
		ISpread<string> FInputFiltered;
		
		[Output("Bin Size")]
		ISpread<int> FBinSize;
		
		[Output("IP Address")]
		ISpread<string> FIpAddress;
		
		[Output("Touch Count")]
		ISpread<int> FTouchCount;
		
		[Output("Is Touching")]
		ISpread<bool> FIsTouching;
		
		[Output("Bang on IP")]
		ISpread<bool> FIP;
		
		[Output("Bang on Init")]
		ISpread<bool> FInit;
		
		[Output("Bang on Volume Press")]
		ISpread<bool> FBangOnVolume;
		
		[Output("Fantastick Version")]
		ISpread<string> FVersion;
		
		[Output("Bang on Touch Begin")]
		ISpread<bool> FTouchBegin;
		
		[Output("Bang on Touch Move")]
		ISpread<bool> FTouchMove;
		
		[Output("Bang on Touch End")]
		ISpread<bool> FTouchEnd;
		
		[Output("Touch Position X")]
		ISpread<double> FTouchX;
		
		[Output("Touch Position Y")]
		ISpread<double> FTouchY;
		
		[Input("Watch Command", DefaultString = "")]
		ISpread<string> fWatchCommand;

		[Output("Bang on Watch Found")]
		ISpread<bool> FWatchFound;
		
		[Output("Watch Output")]
		ISpread<string> FWatchOutput;
		
		[Input("Watch2 Command", DefaultString = "")]
		ISpread<string> fWatch2Command;

		[Output("Bang on Watch2 Found")]
		ISpread<bool> FWatch2Found;
		
		[Output("Watch2 Output")]
		ISpread<string> FWatch2Output;
		
		[Input("Watch3 Command", DefaultString = "")]
		ISpread<string> fWatch3Command;

		[Output("Bang on Watch3 Found")]
		ISpread<bool> FWatch3Found;
		
		[Output("Watch3 Output")]
		ISpread<string> FWatch3Output;
		
		[Input("Watch4 Command", DefaultString = "")]
		ISpread<string> fWatch4Command;

		[Output("Bang on Watch4 Found")]
		ISpread<bool> FWatch4Found;
		
		[Output("Watch4 Output")]
		ISpread<string> FWatch4Output;
		
		[Input("Watch5 Command", DefaultString = "")]
		ISpread<string> fWatch5Command;

		[Output("Bang on Watch5 Found")]
		ISpread<bool> FWatch5Found;
		
		[Output("Watch5 Output")]
		ISpread<string> FWatch5Output;
		
		[Import()]
		ILogger FLogger;
		#endregion fields & pins

		private const int MAX_TOUCHES = 5;
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			int slices = 0;
			int idx;
			
			FBinSize.SliceCount = fInput.SliceCount;
			FOutput.SliceCount = 99;
			
			FTouchBegin.SliceCount =
			FTouchMove.SliceCount = 
			FTouchEnd.SliceCount = 
			FTouchX.SliceCount = 
				FTouchY.SliceCount = 
				FIsTouching.SliceCount = MAX_TOUCHES;
			
			// reset bangs
			for (int i = 0; i<MAX_TOUCHES; i++) {
				FTouchBegin[i] = 
				FTouchMove[i] =
				FTouchEnd[i] = false;
			}
			FInit[0] = false;
			FIP[0] = false;
			FBangOnVolume[0] = false;
			FWatchFound[0] = false;
			FWatch2Found[0] = false;
			FWatch3Found[0] = false;
			FWatch4Found[0] = false;
			FWatch5Found[0] = false;
			
			//log("vvvvvvvvvvvvv");
			
			FInputFiltered.SliceCount = 0;
			
			for (int i = 0; i < fInput.SliceCount; i++) {
				
				string[] words = fInput[i].Split(' ');
				//log("-------");
				
				
				for (int j = 0; j<words.Length; j++) {
					//log((slices+j) + "> "+  words[j]);
					FOutput[slices+j] = words[j];
				}
				
				string cmd = words[0];
				
				bool f = false;
				for (int j=0; j<fFilters.SliceCount && !f; j++) {
					if (cmd.Equals(fFilters[j])) {
						f = true;
					}
				}
				
				//if (cmd != "A" && cmd != "IP" && cmd != "FantaStick") {
				if (!f) {
					FInputFiltered.SliceCount++;
					FInputFiltered[ FInputFiltered.SliceCount-1 ] = fInput[i];
				}
				
				if (cmd.Equals(fWatchCommand[0])) {
					FWatchFound[0] = true;
					FWatchOutput.SliceCount = words.Length;
					for (int j = 0; j<words.Length; j++) {
						FWatchOutput[j] = words[j];
					}
				} else if (cmd.Equals(fWatch2Command[0])) {
					FWatch2Found[0] = true;
					FWatch2Output.SliceCount = words.Length;
					for (int j = 0; j<words.Length; j++) {
						FWatch2Output[j] = words[j];
					}
				} else if (cmd.Equals(fWatch3Command[0])) {
					FWatch3Found[0] = true;
					FWatch3Output.SliceCount = words.Length;
					for (int j = 0; j<words.Length; j++) {
						FWatch3Output[j] = words[j];
					}
				} else if (cmd.Equals(fWatch4Command[0])) {
					FWatch4Found[0] = true;
					FWatch4Output.SliceCount = words.Length;
					for (int j = 0; j<words.Length; j++) {
						FWatch4Output[j] = words[j];
					}
				} else if (cmd.Equals(fWatch5Command[0])) {
					FWatch5Found[0] = true;
					FWatch5Output.SliceCount = words.Length;
					for (int j = 0; j<words.Length; j++) {
						FWatch5Output[j] = words[j];
					}
				}
				
				switch (cmd) {
					case "X":	// # of fingers touching screen
						FTouchCount[0] = Convert.ToInt16(words[1]);
						break;
						
					case "B":	// finger touch begin
						idx = Convert.ToInt16(words[3])-1;
						FIsTouching[idx] = true;
						FTouchBegin[idx] = true;
						//FTouchMove[idx] = false;
						//FTouchEnd[idx] = false;
						FTouchX[idx] = Convert.ToInt16(words[1]);
						FTouchY[idx] = Convert.ToInt16(words[2]);
						break;
						
					case "M":	// finger already touching moved
						idx = Convert.ToInt16(words[3])-1;
						FIsTouching[idx] = true;
						//FTouchBegin[idx] = false;
						FTouchMove[idx] = true;
						//FTouchEnd[idx] = false;
						FTouchX[idx] = Convert.ToInt16(words[1]);
						FTouchY[idx] = Convert.ToInt16(words[2]);
						break;
						
					case "E":	// finger touch end
						idx = Convert.ToInt16(words[3])-1;
						FIsTouching[idx] = false;
						//FTouchBegin[idx] = false;
						//FTouchMove[idx] = false;
						FTouchEnd[idx] = true;
						FTouchX[idx] = Convert.ToInt16(words[1]);
						FTouchY[idx] = Convert.ToInt16(words[2]);
						break;
						
					case "IP":
						FIpAddress[0] = words[1];
						FIP[0] = true;
						break;
					
					case "V":
						FBangOnVolume[0] = true;
						break;
					
					case "FantaStick":
						if (words[1] == "init") FInit[0] = true;
						FVersion[0] = words[2];
						break;
				}
				
				slices += words.Length;
				FBinSize[i] = words.Length;
			}
			
			FOutput.SliceCount = slices;
			
		}
		
		private void log(string s) {
			FLogger.Log(LogType.Debug, s);
		}
	}
}
